package com.huimv.eartag2.manage2.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.manage2.mapper.BasePigpenMapper;
import com.huimv.eartag2.manage2.pojo.BaseFarm;
import com.huimv.eartag2.manage2.pojo.BasePigpen;
import com.huimv.eartag2.manage2.pojo.EartagDeviceRegister;
import com.huimv.eartag2.manage2.mapper.EartagDeviceRegisterMapper;
import com.huimv.eartag2.manage2.service.EartagDeviceRegisterService;
import com.huimv.eartag2.manage2.service.IEartagDeviceRegisterService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.eartag2.manage2.utils.IDUtil;
import com.huimv.eartag2.manage2.utils.ResultUtil;
import com.huimv.eartag2.manage2.vo.EartagDeviceRegisterVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2022-07-20
 */
@Service
public class EartagDeviceRegisterServiceImpl extends ServiceImpl<EartagDeviceRegisterMapper, EartagDeviceRegister> implements EartagDeviceRegisterService {

    @Resource
    private EartagDeviceRegisterMapper eartagDeviceRegisterMapper;

    @Resource
    private BasePigpenMapper basePigpenMapper;

    @Autowired
    private ResultUtil resultUtil;

    @Autowired
    private IDUtil idUtil;



    @Override
    public Result set(List<Map<String, String>> mapList) {
        ArrayList<Integer> ids = new ArrayList<>();
        for (Map<String, String> map : mapList) {
            String id = map.get("id");
            ids.add(Integer.valueOf(id));
        }
        String pigpenId = mapList.get(0).get("pigpenId");
        String unitId = mapList.get(0).get("unitId");
        //批量更新
        for (Integer id : ids) {
            UpdateWrapper<EartagDeviceRegister> queryWrapper = new UpdateWrapper<>();
            queryWrapper.set("pigpen_id",pigpenId).set("unit_id",unitId).eq("id",id);
            eartagDeviceRegisterMapper.update(null,queryWrapper);
        }
        return resultUtil.result(1);
    }

    @Override
    public Result remove(String ids) {
        List<Integer> convert = idUtil.convert(ids);
        return resultUtil.result(eartagDeviceRegisterMapper.deleteBatchIds(convert));
    }

    @Override
    public Result update(Map<String, String> maps) {
        String id = maps.get("id");

        String chipId = maps.get("chipId");
        String deviceCode = maps.get("deviceCode");
        String deviceAlias = maps.get("deviceAlias");
        String activeStatus = maps.get("activeStatus");
        String deviceStatus = maps.get("deviceStatus");

        String remark = maps.get("remark");
        EartagDeviceRegister eartagDeviceRegister = eartagDeviceRegisterMapper.selectById(id);
        eartagDeviceRegister.setChipId(chipId);
        eartagDeviceRegister.setDeviceAlias(deviceAlias);
        eartagDeviceRegister.setDeviceCode(deviceCode);
        eartagDeviceRegister.setActiveStatus(Integer.valueOf(activeStatus));
        eartagDeviceRegister.setRemark(remark);
        eartagDeviceRegister.setDeviceStatus(Integer.valueOf(deviceStatus));
        return resultUtil.result(eartagDeviceRegisterMapper.updateById(eartagDeviceRegister));
    }

    @Override
    public Result list(Map<String, String> paramsMap) {
        //编码，栋舍，单元，别名，生存状态，活动状态
        String deviceCode = paramsMap.get("deviceCode");
        String pigpenId = paramsMap.get("pigpenId");
        String unitId = paramsMap.get("unitId");
        String deviceAlias = paramsMap.get("deviceAlias");
        String deviceStatus = paramsMap.get("deviceStatus");
        String activeStatus = paramsMap.get("activeStatus");
        String pageSize = paramsMap.get("pageSize");
        String pageNo = paramsMap.get("pageNo");
        String farmId = paramsMap.get("farmId");

        if (StringUtils.isBlank(pageNo)){
            pageNo="1";
        }
        if (StringUtils.isBlank(pageSize)){
            pageSize="20";
        }
        QueryWrapper<EartagDeviceRegister> wrapper = new QueryWrapper<>();

        if (StringUtils.isBlank(unitId)){
            wrapper.eq(StringUtils.isNotBlank(pigpenId),"pigpen_id",pigpenId);
        }else {
            wrapper.eq("unit_id",unitId);
        }
        wrapper.like(StringUtils.isNotBlank(deviceCode),"device_code",deviceCode);
        wrapper.like(StringUtils.isNotBlank(deviceAlias),"device_alias",deviceAlias);
        wrapper.like(StringUtils.isNotBlank(deviceStatus),"device_status",deviceStatus);
        wrapper.like(StringUtils.isNotBlank(activeStatus),"active_status",activeStatus);
        wrapper.eq("farm_id",farmId);
        Page<EartagDeviceRegister> page = this.page(new Page<>(Long.parseLong(pageNo), Long.parseLong(pageSize)), wrapper);
        Page<EartagDeviceRegisterVo> eartagDeviceRegisterVoPage = new Page<>();
        //复制
        List<EartagDeviceRegisterVo> listVo = new ArrayList();
        BeanUtil.copyProperties(page,eartagDeviceRegisterVoPage);
        for (EartagDeviceRegister record : page.getRecords()) {
            EartagDeviceRegisterVo eartagDeviceRegisterVo = new EartagDeviceRegisterVo();
            BeanUtil.copyProperties(record,eartagDeviceRegisterVo);
            if (record.getPigpenId() != null){
                eartagDeviceRegisterVo.setPigpenName(basePigpenMapper.selectById(record.getPigpenId()).getPigpenName());
            }

            if (record.getPigpenId() != null){
                eartagDeviceRegisterVo.setUnitName(basePigpenMapper.selectById(record.getUnitId()).getPigpenName());
            }

            listVo.add(eartagDeviceRegisterVo);
        }

        eartagDeviceRegisterVoPage.setRecords(listVo);

        return new Result(ResultCode.SUCCESS,eartagDeviceRegisterVoPage);
    }

    @Override
    public Result add(Map<String, String> mapList) {
        String farmId = mapList.get("farmId");
        String chipId = mapList.get("chipId");
        String deviceAlias = mapList.get("deviceAlias");
        String activeStatus = mapList.get("activeStatus");
        String deviceStatus = mapList.get("deviceStatus");
        String remark = mapList.get("remark");

        String deviceCode = (String)getDeviceCode(farmId).getData();
        EartagDeviceRegister eartagDeviceRegister = new EartagDeviceRegister();
        eartagDeviceRegister.setChipId(chipId);
        eartagDeviceRegister.setDeviceAlias(deviceAlias);
        eartagDeviceRegister.setDeviceCode(deviceCode);
        eartagDeviceRegister.setActiveStatus(Integer.valueOf(activeStatus));
        eartagDeviceRegister.setRemark(remark);
        eartagDeviceRegister.setDeviceStatus(Integer.valueOf(deviceStatus));
        eartagDeviceRegister.setFarmId(farmId);
        return resultUtil.result(eartagDeviceRegisterMapper.insert(eartagDeviceRegister));
    }

    @Override
    public Result cancel(String ids) {
        List<Integer> convert = idUtil.convert(ids);
        for (Integer id : convert) {
            UpdateWrapper<EartagDeviceRegister> queryWrapper = new UpdateWrapper<>();
            queryWrapper.set("pigpen_id",null).set("unit_id",null).eq("id",id);
            eartagDeviceRegisterMapper.update(null,queryWrapper);
        }
        return resultUtil.result(1);
    }

    @Override
    public Result getDeviceCode(String farmCode) {
        QueryWrapper<EartagDeviceRegister> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id",farmCode);
        List<EartagDeviceRegister> eartagDeviceRegisters = eartagDeviceRegisterMapper.selectList(queryWrapper);
        if (null == eartagDeviceRegisters || eartagDeviceRegisters.size() == 0){
            return new Result(ResultCode.SUCCESS,farmCode+"000001");
        }else {
            String maxDeviceCode = eartagDeviceRegisters.stream().max(Comparator.comparing(EartagDeviceRegister::getDeviceCode)).get().getDeviceCode();
            String substring = maxDeviceCode.substring(maxDeviceCode.length() - 6);
            int code = Integer.parseInt(substring)+1;
            String format = String.format("%6d", code);
            return new Result(ResultCode.SUCCESS,farmCode+format);
        }
    }
}
